=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2011-01-12 22:21:05 +0000
+++ bzrlib/tests/test_http.py	2011-02-03 05:46:08 +0000
@@ -1089,6 +1089,16 @@
         self.overrideEnv('http_proxy', 'host:1234')
         self.assertRaises(errors.InvalidURL, self._proxied_request)
 
+    def test_evaluate_proxy_bypass(self):
+        """Test matching of no_proxy etc"""
+        handler = _urllib2_wrappers.ProxyHandler()
+        # https://bugs.launchpad.net/bzr/+bug/586341
+        self.assertEquals(
+            None,
+            handler.evaluate_proxy_bypass(
+                'example.com',
+                ','))
+
 
 class TestProxyHttpServer(http_utils.TestCaseWithTwoWebservers):
     """Tests proxy server.

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2011-01-12 21:22:20 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2011-02-03 05:46:08 +0000
@@ -944,6 +944,22 @@
     def proxy_bypass(self, host):
         """Check if host should be proxied or not"""
         no_proxy = self.get_proxy_env_var('no', default_to=None)
+        result = self.evaluate_proxy_bypass(host, no_proxy)
+        if result is None:
+            # Nevertheless, there are platform-specific ways to
+            # ignore proxies...
+            return urllib.proxy_bypass(host)
+        else:
+            return result
+
+    def evaluate_proxy_bypass(self, host, no_proxy):
+        """Check the host against a comma-separated no_proxy list.
+
+        :param host: host:port being requested
+        :param no_proxy: comma-separated list of hosts to access directly.
+        :returns: True to skip the proxy, False not to, or None to
+            leave it to urllib.
+        """
         if no_proxy is None:
             return False
         hhost, hport = urllib.splitport(host)
@@ -952,6 +968,9 @@
         # are fuzzy (to say the least). We try to allow most
         # commonly seen values.
         for domain in no_proxy.split(','):
+            domain = domain.strip()
+            if domain == '':
+                continue
             dhost, dport = urllib.splitport(domain)
             if hport == dport or dport is None:
                 # Protect glob chars
@@ -959,10 +978,7 @@
                 dhost = dhost.replace("*", r".*")
                 dhost = dhost.replace("?", r".")
                 if re.match(dhost, hhost, re.IGNORECASE):
-                    return True
-        # Nevertheless, there are platform-specific ways to
-        # ignore proxies...
-        return urllib.proxy_bypass(host)
+                    return True        
 
     def set_proxy(self, request, type):
         if self.proxy_bypass(request.get_host()):

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-01-27 17:45:24 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-02-03 05:46:08 +0000
@@ -71,6 +71,10 @@
 * ``launchpadlib`` doesn't provide the ``uris`` module in some old versions.
   (Vincent Ladeuil, #706835)
 
+* Empty entries in the ``NO_PROXY`` variable are no longer treated as matching
+  every host.
+  (Martin Pool, #586341)
+
 * Plugins incompatible with the current version of bzr no longer produce a
   warning on every command invocation.  Instead, a message is shown by
   ``bzr plugins`` and in crash reports.

